@@ -169,6 +169,7 @@ class PowerPlotXYWidget(QWidget):
                     beam.rays[:, 15] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 15]
                     beam.rays[:, 16] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 16]
                     beam.rays[:, 17] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 17]
+                    beam.rays[:, 9]  = 1
                 else:
                     incident_rays = previous_beam._beam.rays
                     transmitted_rays = current_beam._beam.rays
@@ -214,7 +215,7 @@ class PowerPlotXYWidget(QWidget):
 
         good = numpy.where(ticket_initial["histogram"] > 0)
 
-        efficiency_incident = numpy.zeros(len(ticket_incident["histogram"]))
+        efficiency_incident        = numpy.zeros(len(ticket_incident["histogram"]))
         efficiency_incident[good]  = ticket_incident["histogram"][good] / ticket_initial["histogram"][good]
 
         incident_power_shadow = initial_power_shadow * efficiency_incident
@@ -235,12 +236,14 @@ class PowerPlotXYWidget(QWidget):
         ticket = beam.histo1(11, xrange=energy_range, nbins=nbins_interpolation, nolost=1, ref=0) # number of rays per bin
         good = numpy.where(ticket["histogram"] > 0)
 
-        final_power_per_ray = numpy.zeros(len(final_power_shadow))
+        final_power_per_ray       = numpy.zeros(len(final_power_shadow))
         final_power_per_ray[good] = final_power_shadow[good] / ticket["histogram"][good]
 
-        go = numpy.where(shadow_beam._beam.rays[:, 9] == 1)
+        #go = numpy.where(shadow_beam._beam.rays[:, 9] == 1)
+        go = numpy.where(beam.rays[:, 9] == 1)
 
-        rays_energy = ShadowPhysics.getEnergyFromShadowK(shadow_beam._beam.rays[go, 10])
+        #rays_energy = ShadowPhysics.getEnergyFromShadowK(shadow_beam._beam.rays[go, 10])
+        rays_energy = ShadowPhysics.getEnergyFromShadowK(beam.rays[go, 10])
 
         ticket = beam.histo2(var_x, var_y, nbins_h=nbins_h, nbins_v=nbins_v, xrange=xrange, yrange=yrange, nolost=1, ref=0)
 
@@ -345,6 +348,7 @@ class PowerPlotXYWidget(QWidget):
                     beam.rays[:, 15] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 15]
                     beam.rays[:, 16] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 16]
                     beam.rays[:, 17] = lost_rays_in_previous_beam[lost_that_were_good_rays_cursor][:, 17]
+                    beam.rays[:, 9]  = 1
                 else:
                     incident_rays = previous_beam._beam.rays
                     transmitted_rays = current_beam._beam.rays
